@@ -1,21 +1,24 @@
-/* Copyright (C) 2015-2017 Jason A. Donenfeld <Jason@zx2c4.com>. All Rights Reserved. */
+/* SPDX-License-Identifier: GPL-2.0
+ *
+ * Copyright (C) 2015-2018 Jason A. Donenfeld <Jason@zx2c4.com>. All Rights Reserved.
+ */
 
 #include "cookie.h"
 #include "peer.h"
 #include "device.h"
 #include "messages.h"
 #include "ratelimiter.h"
+#include "timers.h"
 #include "crypto/blake2s.h"
 #include "crypto/chacha20poly1305.h"
 
-#include <linux/jiffies.h>
 #include <net/ipv6.h>
 #include <crypto/algapi.h>
 
 void cookie_checker_init(struct cookie_checker *checker, struct wireguard_device *wg)
 {
 	init_rwsem(&checker->secret_lock);
-	checker->secret_birthdate = get_jiffies_64();
+	checker->secret_birthdate = ktime_get_boot_fast_ns();
 	get_random_bytes(checker->secret, NOISE_HASH_LEN);
 	checker->device = wg;
 }
@@ -27,24 +30,23 @@ static const u8 cookie_key_label[COOKIE_KEY_LABEL_LEN] = "cookie--";
 static void precompute_key(u8 key[NOISE_SYMMETRIC_KEY_LEN], const u8 pubkey[NOISE_PUBLIC_KEY_LEN], const u8 label[COOKIE_KEY_LABEL_LEN])
 {
 	struct blake2s_state blake;
+
 	blake2s_init(&blake, NOISE_SYMMETRIC_KEY_LEN);
 	blake2s_update(&blake, label, COOKIE_KEY_LABEL_LEN);
 	blake2s_update(&blake, pubkey, NOISE_PUBLIC_KEY_LEN);
 	blake2s_final(&blake, key, NOISE_SYMMETRIC_KEY_LEN);
 }
 
+/* Must hold peer->handshake.static_identity->lock */
 void cookie_checker_precompute_device_keys(struct cookie_checker *checker)
 {
-	down_read(&checker->device->static_identity.lock);
 	if (likely(checker->device->static_identity.has_identity)) {
 		precompute_key(checker->cookie_encryption_key, checker->device->static_identity.static_public, cookie_key_label);
 		precompute_key(checker->message_mac1_key, checker->device->static_identity.static_public, mac1_key_label);
-	}
-	else {
+	} else {
 		memset(checker->cookie_encryption_key, 0, NOISE_SYMMETRIC_KEY_LEN);
 		memset(checker->message_mac1_key, 0, NOISE_SYMMETRIC_KEY_LEN);
 	}
-	up_read(&checker->device->static_identity.lock);
 }
 
 void cookie_checker_precompute_peer_keys(struct wireguard_peer *peer)
@@ -75,9 +77,9 @@ static void make_cookie(u8 cookie[COOKIE_LEN], struct sk_buff *skb, struct cooki
 {
 	struct blake2s_state state;
 
-	if (!time_is_after_jiffies64(checker->secret_birthdate + COOKIE_SECRET_MAX_AGE)) {
+	if (has_expired(checker->secret_birthdate, COOKIE_SECRET_MAX_AGE)) {
 		down_write(&checker->secret_lock);
-		checker->secret_birthdate = get_jiffies_64();
+		checker->secret_birthdate = ktime_get_boot_fast_ns();
 		get_random_bytes(checker->secret, NOISE_HASH_LEN);
 		up_write(&checker->secret_lock);
 	}
@@ -139,7 +141,7 @@ void cookie_add_mac_to_packet(void *message, size_t len, struct wireguard_peer *
 	up_write(&peer->latest_cookie.lock);
 
 	down_read(&peer->latest_cookie.lock);
-	if (peer->latest_cookie.is_valid && time_is_after_jiffies64(peer->latest_cookie.birthdate + COOKIE_SECRET_MAX_AGE - COOKIE_SECRET_LATENCY))
+	if (peer->latest_cookie.is_valid && !has_expired(peer->latest_cookie.birthdate, COOKIE_SECRET_MAX_AGE - COOKIE_SECRET_LATENCY))
 		compute_mac2(macs->mac2, message, len, peer->latest_cookie.cookie);
 	else
 		memset(macs->mac2, 0, COOKIE_LEN);
@@ -162,31 +164,30 @@ void cookie_message_create(struct message_handshake_cookie *dst, struct sk_buff
 void cookie_message_consume(struct message_handshake_cookie *src, struct wireguard_device *wg)
 {
 	u8 cookie[COOKIE_LEN];
-	struct index_hashtable_entry *entry;
+	struct wireguard_peer *peer = NULL;
 	bool ret;
 
-	entry = index_hashtable_lookup(&wg->index_hashtable, INDEX_HASHTABLE_HANDSHAKE | INDEX_HASHTABLE_KEYPAIR, src->receiver_index);
-	if (unlikely(!entry))
+	if (unlikely(!index_hashtable_lookup(&wg->index_hashtable, INDEX_HASHTABLE_HANDSHAKE | INDEX_HASHTABLE_KEYPAIR, src->receiver_index, &peer)))
 		return;
 
-	down_read(&entry->peer->latest_cookie.lock);
-	if (unlikely(!entry->peer->latest_cookie.have_sent_mac1)) {
-		up_read(&entry->peer->latest_cookie.lock);
+	down_read(&peer->latest_cookie.lock);
+	if (unlikely(!peer->latest_cookie.have_sent_mac1)) {
+		up_read(&peer->latest_cookie.lock);
 		goto out;
 	}
-	ret = xchacha20poly1305_decrypt(cookie, src->encrypted_cookie, sizeof(src->encrypted_cookie), entry->peer->latest_cookie.last_mac1_sent, COOKIE_LEN, src->nonce, entry->peer->latest_cookie.cookie_decryption_key);
-	up_read(&entry->peer->latest_cookie.lock);
+	ret = xchacha20poly1305_decrypt(cookie, src->encrypted_cookie, sizeof(src->encrypted_cookie), peer->latest_cookie.last_mac1_sent, COOKIE_LEN, src->nonce, peer->latest_cookie.cookie_decryption_key);
+	up_read(&peer->latest_cookie.lock);
 
 	if (ret) {
-		down_write(&entry->peer->latest_cookie.lock);
-		memcpy(entry->peer->latest_cookie.cookie, cookie, COOKIE_LEN);
-		entry->peer->latest_cookie.birthdate = get_jiffies_64();
-		entry->peer->latest_cookie.is_valid = true;
-		entry->peer->latest_cookie.have_sent_mac1 = false;
-		up_write(&entry->peer->latest_cookie.lock);
+		down_write(&peer->latest_cookie.lock);
+		memcpy(peer->latest_cookie.cookie, cookie, COOKIE_LEN);
+		peer->latest_cookie.birthdate = ktime_get_boot_fast_ns();
+		peer->latest_cookie.is_valid = true;
+		peer->latest_cookie.have_sent_mac1 = false;
+		up_write(&peer->latest_cookie.lock);
 	} else
 		net_dbg_ratelimited("%s: Could not decrypt invalid cookie response\n", wg->dev->name);
 
 out:
-	peer_put(entry->peer);
+	peer_put(peer);
 }
